from django import forms
from django.contrib.auth import forms as auth_forms
from django.contrib.auth.tokens import default_token_generator
from django.contrib.sites.shortcuts import get_current_site
from django.utils.encoding import force_bytes
from django.utils.http import urlsafe_base64_encode

from benevalibre.utils.forms import CustomTapeformMixin
from benevalibre.utils.utils import get_instance_name

from .models import User


class PasswordFormMixin(CustomTapeformMixin):
    def apply_widget_options(self, field_name):
        if field_name.startswith('new_password'):
            field = self.fields[field_name]
            field.widget.attrs['autocomplete'] = 'new-password'


class AuthenticationForm(CustomTapeformMixin, auth_forms.AuthenticationForm):
    """
    Formulaire d'authentification de l'utilisat⋅eur⋅rice.
    """

    error_messages = {
        'invalid_login': (
            "L'adresse mail et le mot de passe ne correspondent pas, "
            "veuillez réessayer."
        ),
        'inactive': (
            "Le compte est désactivé. Veuillez nous contacter pour plus "
            "d'informations."
        ),
    }


class PasswordResetForm(CustomTapeformMixin, auth_forms.PasswordResetForm):
    """
    Formulaire de demande de réinitialisation du mot de passe.
    """

    email = forms.EmailField(label="Adresse mail", max_length=254)

    def save(
        self,
        subject_template_name='registration/password_reset_subject.txt',
        email_template_name='registration/password_reset_email.html',
        use_https=False,
        token_generator=default_token_generator,
        from_email=None,
        request=None,
        html_email_template_name=None,
        extra_email_context=None,
    ):
        """
        Rewrite auth_forms' code since django doesnt support idna.
        Generate a one-use only link for resetting password and send it to the
        user.
        """
        email = self.cleaned_data["email"]
        instance_name = get_instance_name(request)
        domain = get_current_site(request).domain
        email_field_name = User.get_email_field_name()
        for user in self.get_users(email):
            user_email = getattr(user, email_field_name)
            context = {
                'email': user_email,
                'domain': domain,
                'instance_name': instance_name,
                'uid': urlsafe_base64_encode(force_bytes(user.pk)),
                'user': user,
                'token': token_generator.make_token(user),
                'protocol': 'https' if use_https else 'http',
                **(extra_email_context or {}),
            }
            self.send_mail(
                subject_template_name,
                email_template_name,
                context,
                from_email,
                user_email,
                html_email_template_name=html_email_template_name,
            )


class SetPasswordForm(PasswordFormMixin, auth_forms.SetPasswordForm):
    """
    Formulaire de réinitialisation du mot de passe, sans avoir à saisir
    l'ancien.
    """


class PasswordChangeForm(PasswordFormMixin, auth_forms.PasswordChangeForm):
    """
    Formulaire de changement de mot de passe avec l'ancien.
    """


class UserUpdateForm(CustomTapeformMixin, forms.ModelForm):
    """
    Formulaire d'édition du profil utilisat⋅eur⋅rice.
    """

    layout_template = 'accounts/forms/layouts/user.html'

    class Meta:
        model = User
        fields = ('email', 'first_name', 'last_name', 'pseudo', 'avatar')


class RegistrationForm(CustomTapeformMixin, auth_forms.UserCreationForm):
    """
    Formulaire de demande de création d'un compte.
    """

    layout_template = 'accounts/forms/layouts/user_registration.html'

    class Meta:
        model = User
        fields = ('email', 'first_name', 'last_name', 'pseudo')


class ResendActivationForm(CustomTapeformMixin, forms.Form):
    """
    Formulaire de demande d'envoi du lien d'activation du compte.
    """

    email = forms.EmailField(label="Adresse mail", max_length=254)
