from datetime import timedelta

from django.db.models import Count, Sum

from benevalibre.base.tables import format_duration
from benevalibre.benevalo.models import Benevalo

# ----------------------------------------------------------------------------
# Mixin for Statistics pages
# ----------------------------------------------------------------------------


class StatsPageMixin:
    def get_benevalos(self, year=None):
        benevalos = Benevalo.objects.active()
        if year:
            return benevalos.filter(execution_date__year=year)
        return benevalos

    def get_active_years(self):
        active_years = (
            self.get_benevalos()
            .order_by('execution_date__year')
            .values('execution_date__year')
            .distinct()
        )
        return [year.get('execution_date__year') for year in active_years]

    def get_benevalo_count(self):
        return self.get_benevalos().count()

    def get_total_benevalo_duration(self):
        total = self.get_benevalos().aggregate(total_duration=Sum('duration'))
        return format_duration(total.get('total_duration') or timedelta())

    def get_total_benevalo_distance(self):
        return (
            self.get_benevalos()
            .aggregate(total_distance=Sum('distance'))
            .get('total_distance')
        ) or 0

    def get_benevalo_count_this_year(self, year):
        return self.get_benevalos(year).count()

    def get_benevalo_duration_this_year(self, year):
        total = self.get_benevalos(year).aggregate(
            total_duration=Sum('duration')
        )
        return format_duration(total.get('total_duration') or timedelta())

    def get_benevalo_distance_this_year(self, year):
        return (
            self.get_benevalos(year)
            .aggregate(total_distance=Sum('distance'))
            .get('total_distance')
        ) or 0

    def get_benevalo_data_year_by_year(self):
        return {
            value: {
                'count': self.get_benevalo_count_this_year(value),
                'duration': self.get_benevalo_duration_this_year(value),
                'distance': self.get_benevalo_distance_this_year(value),
            }
            for value in self.get_active_years()
        }

    def get_object_count(self, year, _filter):
        return (
            self.get_benevalos(year)
            .filter(**_filter)
            .aggregate(total=Count('category'))
            .get('total')
        )

    def get_user_count_for_object(self, year, _filter):
        return (
            self.get_benevalos(year)
            .filter(**_filter)
            .values('user')
            .distinct()
            .aggregate(total=Count('user'))
            .get('total')
        )

    def get_total_duration_for_object(self, year, _filter):
        total = (
            self.get_benevalos(year)
            .filter(**_filter)
            .aggregate(total_duration=Sum('duration'))
        )
        return format_duration(total.get('total_duration') or timedelta())

    def get_total_distance_for_object(self, year, _filter):
        return (
            self.get_benevalos(year)
            .filter(**_filter)
            .aggregate(total_distance=Sum('distance'))
            .get('total_distance')
        ) or 0

    def get_page_context(self):
        context = super().get_page_context()
        context.update(
            {
                'nbBenevalo': self.get_benevalo_count(),
                'totalDuration': self.get_total_benevalo_duration(),
                'totalDistance': self.get_total_benevalo_distance(),
                'statsBenevaloByYear': self.get_benevalo_data_year_by_year(),
                'activeYears': self.get_active_years(),
            }
        )
        return context
