from datetime import datetime, timedelta, timezone

from django.core.exceptions import ValidationError
from django.db import models

import reversion

from benevalibre.utils.mixins import HTMLDocString
from benevalibre.utils.models import AbstractRole, AbstractTaxonomy
from benevalibre.utils.validators import validate_after2k, validate_before1y

from .validators import validate_is_pdf


@reversion.register()
class InstanceCategory(HTMLDocString, AbstractTaxonomy):
    """
    Les catégories servent à qualifier les actions de bénévolat. Mais
    elles sont propres à chaque association, qui peut personnaliser sa
    liste de catégories.

    Afin de rendre possible des statistiques inter-associatives, elles sont
    donc chacune attachées à une « catégorie d'instance ».

    Par conséquent il n'est pas recommandé de modifier ces groupes sans
    concertation préalable avec les fédérations de mouvements associatifs.
    """

    class Meta:
        verbose_name = "catégorie d'instance"
        verbose_name_plural = "catégories d'instance"


class AbstractLevel(AbstractTaxonomy):
    class Meta:
        abstract = True


@reversion.register()
class DefaultLevel(HTMLDocString, AbstractLevel):
    """
    Les « niveaux par défaut » sont crées et affectés
    par défaut à toute nouvelle association inscrite sur la plateforme.

    Les niveaux de bénévolat permettent pour ceux qui souhaitent de
    caractériser le niveau d'expertise que l'action mobilise. C'est utile
    par exemple pour les associations qui souhaitent moduler la
    valorisation en fonction de niveaux d'expertise différents.
    """

    class Meta:
        verbose_name = "niveau par défaut"
        verbose_name_plural = "niveaux par défaut"


class AbstractCategory(AbstractTaxonomy):
    class Meta:
        abstract = True

    instance_category = models.ForeignKey(
        InstanceCategory,
        verbose_name="catégorie d'instance",
        on_delete=models.PROTECT,
    )


@reversion.register()
class DefaultCategory(HTMLDocString, AbstractCategory):
    """
    Les « catégories par défaut » sont crées et affectés
    par défaut à toute nouvelle association inscrite sur la plateforme.

    Afin de rendre possible des statistiques inter-associatives, elles
    sont chacune attachées à une « catégorie d'instance ».

    Ainsi, chaque association peut personnaliser sa liste de catégories.
    """

    class Meta:
        verbose_name = "catégorie par défaut"
        verbose_name_plural = "catégories par défaut"


class DefaultRole(HTMLDocString, AbstractRole):
    """
    Les « rôles par défaut » sont crées et affectés
    par défaut à toute nouvelle association inscrite sur la plateforme.

    Les rôles définissent les permissions dont jouit chaque bénévole.
    Ces permissions sont, par ordre d'importance croissante :

    * Liste les utilisat⋅eurs⋅rices : donne au ou à la bénévole la possibilité
      de voir qui sont les autres bénévoles engagés dans l'association.

    * Délègue ses permissions : donne au ou à la bénévole la possibilité de
      gérer les rôles des bénévoles et de transférer son rôle aux autres
      bénévoles. Ce rôle implique de lister les autres bénévoles engagés dans
      l'association.

    * Gère le bénévolat : donne au ou à la bénévole la possibilité de modérer,
      saisir, corriger ou effacer le bénévolat des autres bénévoles. Ce rôle
      implique de lister les autres bénévoles engagés dans l'association.

    * Gère les niveaux, projets, catégories : donne respectivement au ou à la
      bénévole la capacité de gérer respectivement les niveaux, les projets et
      les catégories.

    * Gère les rôles : donne au ou à la bénévole la capacité de gérer les
      rôles. Ce rôle permet un accès total à tous les rôles de l'association.

    * Gère l'association␣: donne au ou à la bénévole toutes les permissions
      susmentionnées, ainsi que le droit de modifier la totalité des attributs
      de l'association.

    Le « rôle par défaut dans l’association » est le rôle associé aux bénévole
    lorsque ceux-ci rejoignent l'association.
    """

    class Meta:
        verbose_name = "rôle par défaut"
        verbose_name_plural = "rôles par défaut"

    def save(self, *args, **kwargs):
        self.clean()
        super().save(*args, **kwargs)

    def clean(self):
        if (
            self.new_comers
            and DefaultRole.objects.filter(
                ~models.Q(id=self.id), new_comers=True
            ).exists()
        ):
            raise ValidationError(
                "Un autre rôle possède déjà l'attribut « rôle par défaut ». "
                "Il ne peut n'y en avoir qu'un."
            )
        elif (
            not self.new_comers
            and not DefaultRole.objects.filter(
                ~models.Q(id=self.id), new_comers=True
            ).exists()
        ):
            raise ValidationError(
                "Aucun autre rôle ne possède l'attribut « rôle par défaut ». "
                "Vous devez en disposer d'un pour permettre l'inscription d'un "
                "bénévole."
            )


class TermsOfUse(models.Model):
    """
    La charte d'utilisation de la plateforme.

    Une fois définie elle apparait à tous les utilisateurs via le menu
    principal de la plateforme. Le format attendu est un pdf.

    Pour mettre à jour la charte il suffit de téléverser un nouveau document.
    """

    class Meta:
        verbose_name = "charte d'utilisation"
        verbose_name_plural = "chartes d'utilisation"

    file = models.FileField(
        'Charte',
        upload_to='terms',
        validators=[validate_is_pdf],
        help_text="Format attendu : pdf",
    )


class InstanceMessage(models.Model):
    """
    Message publié par un·e administrateur·ice pour prévenir les usager·es d'une
    instance, par exemple lors d'une mise à jour où d'un évènement.
    """

    class Meta:
        verbose_name = "message d'instance"
        verbose_name_plural = "messages d'instance"

    class TargetType(models.TextChoices):
        EVERYONE = 'ALL', 'Tout le monde'
        MANAGERS = 'MNG', 'Les managers d\'asso'

    def default_expiration_date():
        return datetime.today() + timedelta(days=1)

    content = models.TextField("contenu")
    author = models.ForeignKey(
        'accounts.User',
        null=True,
        on_delete=models.SET_NULL,
        verbose_name="auteur·e",
    )

    broadcast_date = models.DateTimeField(
        "date de diffusion",
        default=datetime.today,
        validators=[validate_after2k, validate_before1y],
    )

    expiration_date = models.DateTimeField(
        "date d'expiration",
        null=True,
        default=default_expiration_date,
        validators=[validate_after2k, validate_before1y],
    )

    target = models.CharField(
        'cible',
        max_length=3,
        choices=TargetType.choices,
        default=TargetType.EVERYONE,
    )

    @property
    def name(self):
        return self.content[0:20] + "…"

    def __str__(self):
        return self.name

    def is_target(self, user):
        if self.target == self.TargetType.EVERYONE:
            return True

        if self.target == self.TargetType.MANAGERS:
            return user.is_authenticated and user.is_manager

    def is_on_time(self):
        now = datetime.now(timezone.utc)
        return now >= self.broadcast_date and now < self.expiration_date

    def should_be_displayed(self, user):
        return self.is_on_time() and self.is_target(user)
