import re

from django.conf import settings
from django.test.html import parse_html

RE_DELETE_BTN = re.compile(r"^[\s]*(Supprimer|Refuser)[\s]*$", re.MULTILINE)


def parse_content(response, html=False):
    """
    Décode et retourne le contenu de la réponse, en le transformant en
    objet de structure Python si `html` vaut `True`.
    """
    if (
        hasattr(response, "render")
        and callable(response.render)
        and not response.is_rendered
    ):
        response.render()
    content = response.content.decode(response.charset)
    return parse_html(content) if html else content


def count_text_in_content(response, text, html=False):
    """
    Retourne le nombre d'occurrence de `text` dans le contenu de la réponse,
    en les analysant en tant que HTML si `html` vaut `True`.

    Cette méthode se base sur `django.test.SimpleTestCase.assertContains()`,
    et pourrait être remplacée après l'intégration de pytest-django#709.
    """
    content = parse_content(response, html=html)
    text = parse_html(text) if html else str(text)
    return content.count(text)


def is_login_view(response):
    """
    Détermine si la vue servie par la réponse est la connexion du compte.
    """
    return response.resolver_match.view_name == settings.LOGIN_URL


class StaffOnlyViewMixin:
    """
    Définis des tests pour vérifier que l'accès à une vue n'est possible
    que pour le super-utilisateur.
    """

    url = None
    title = None

    def setup(self):
        if not self.url or not self.title:
            raise ValueError("url and title are required")

    def test_get_anonymous(self, client):
        response = client.get(self.url, follow=True)
        assert response.status_code == 200
        assert is_login_view(response)

    def test_get_user(self, client, user):
        client.force_login(user)

        response = client.get(self.url, follow=True)
        assert response.status_code == 403
        assert count_text_in_content(response, "Accès refusé")

    def test_get_staff(self, client, superuser):
        client.force_login(superuser)

        response = client.get(self.url, follow=True)
        assert response.status_code == 200
        assert count_text_in_content(response, self.title)


class ManagerOnlyViewMixin(StaffOnlyViewMixin):
    """
    Définis des tests pour vérifier que l'accès à une vue n'est possible
    que pour un dirigeant.
    """

    def test_get_manager(self, client, user, manager):
        client.force_login(user)

        response = client.get(self.url, follow=True)
        assert response.status_code == 200
        assert count_text_in_content(response, self.title)

    def test_get_staff(self, client, superuser):
        client.force_login(superuser)

        response = client.get(self.url, follow=True)
        assert response.status_code == 403
        assert count_text_in_content(response, "Accès refusé")
