from importlib import import_module

from django.urls import include, path

CRUD_CLASSES_MAP = [
    # (route, name, class_suffix)
    ('', 'index', 'Index'),
    ('create/', 'create', 'Create'),
    ('<int:pk>/', 'detail', 'Detail'),
    ('<int:pk>/update/', 'update', 'Update'),
    ('<int:pk>/delete/', 'delete', 'Delete'),
]


def crud_include(
    views_module,
    class_prefix,
    namespace=None,
    classes_map=CRUD_CLASSES_MAP,
):
    """
    Generate URLs of CRUD views for classes located in `views_module` - which
    can be a module or a module name - and starting with `class_prefix`. The
    classes are retrieved following the `classes_map` - a 3-tuple of route,
    name and class suffix - if they are found in the module.

    Example usage:

        urlpatterns = [
            path('my-model/',
                 crud_include('myapp.views', 'MyModel', 'my-model')),
        ]

    In this example, the name to reverse the Create view is: `my-model:create`.
    """
    if isinstance(views_module, str):
        views_module = import_module(views_module)
    pattern_list = []
    for route, name, class_suffix in classes_map:
        try:
            cls = getattr(
                views_module, '{}{}'.format(class_prefix, class_suffix)
            )
        except AttributeError:
            continue
        pattern_list.append(path(route, cls.as_view(), name=name))
    if not pattern_list:
        raise ValueError("Unable to find any CRUD views in this module")
    return include((pattern_list, namespace) if namespace else pattern_list)
